#!/usr/bin/python3
# -*- coding:utf-8 -*-
# @FileName  :__init__.py
# @Time      :2022/9/4 18:11
# @Author    :lovemefan
# @email     :lovemefan@outlook.com
from .client.client import TZPRC_Client
from .server.server import TZRPC_Server

__all__ = ["TZRPC_Server", "TZPRC_Client"]



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: Server.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import Boolean_pb2 as Boolean__pb2
import Bytes_pb2 as Bytes__pb2
import Number_pb2 as Number__pb2
import Numpy_pb2 as Numpy__pb2
import String_pb2 as String__pb2
import Tensor_pb2 as Tensor__pb2

DESCRIPTOR = _descriptor.FileDescriptor(
    name="Server.proto",
    package="tzrpc.proto",
    syntax="proto3",
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n\x0cServer.proto\x12\x0btzrpc.proto\x1a\x0cString.proto\x1a\x0cNumber.proto\x1a\x0b\x42ytes.proto\x1a\rBoolean.proto\x1a\x0bNumpy.proto\x1a\x0cTensor.proto2\x95\x07\n\x08toObject\x12\x36\n\x08toString\x12\x13.tzrpc.proto.String\x1a\x13.tzrpc.proto.String"\x00\x12\x39\n\ttoInteger\x12\x14.tzrpc.proto.Integer\x1a\x14.tzrpc.proto.Integer"\x00\x12\x33\n\x07toFloat\x12\x12.tzrpc.proto.Float\x1a\x12.tzrpc.proto.Float"\x00\x12\x36\n\x08toDouble\x12\x13.tzrpc.proto.Double\x1a\x13.tzrpc.proto.Double"\x00\x12\x39\n\ttoBoolean\x12\x14.tzrpc.proto.Boolean\x1a\x14.tzrpc.proto.Boolean"\x00\x12\x33\n\x07toBytes\x12\x12.tzrpc.proto.Bytes\x1a\x12.tzrpc.proto.Bytes"\x00\x12\x39\n\ttoNdarray\x12\x14.tzrpc.proto.ndarray\x1a\x14.tzrpc.proto.ndarray"\x00\x12\x36\n\x08toTensor\x12\x13.tzrpc.proto.Tensor\x1a\x13.tzrpc.proto.Tensor"\x00\x12<\n\ntoNdarrays\x12\x15.tzrpc.proto.ndarrays\x1a\x15.tzrpc.proto.ndarrays"\x00\x12\x39\n\ttoTensors\x12\x14.tzrpc.proto.Tensors\x1a\x14.tzrpc.proto.Tensors"\x00\x12T\n\x12toIntegerArrayList\x12\x1d.tzrpc.proto.IntegerArrayList\x1a\x1d.tzrpc.proto.IntegerArrayList"\x00\x12N\n\x10toFloatArrayList\x12\x1b.tzrpc.proto.FloatArrayList\x1a\x1b.tzrpc.proto.FloatArrayList"\x00\x12Q\n\x11toDoubleArrayList\x12\x1c.tzrpc.proto.DoubleArrayList\x1a\x1c.tzrpc.proto.DoubleArrayList"\x00\x12T\n\x12toBooleanArrayList\x12\x1d.tzrpc.proto.BooleanArrayList\x1a\x1d.tzrpc.proto.BooleanArrayList"\x00\x62\x06proto3',
    dependencies=[
        String__pb2.DESCRIPTOR,
        Number__pb2.DESCRIPTOR,
        Bytes__pb2.DESCRIPTOR,
        Boolean__pb2.DESCRIPTOR,
        Numpy__pb2.DESCRIPTOR,
        Tensor__pb2.DESCRIPTOR,
    ],
)


_sym_db.RegisterFileDescriptor(DESCRIPTOR)


_TOOBJECT = _descriptor.ServiceDescriptor(
    name="toObject",
    full_name="tzrpc.proto.toObject",
    file=DESCRIPTOR,
    index=0,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_start=113,
    serialized_end=1030,
    methods=[
        _descriptor.MethodDescriptor(
            name="toString",
            full_name="tzrpc.proto.toObject.toString",
            index=0,
            containing_service=None,
            input_type=String__pb2._STRING,
            output_type=String__pb2._STRING,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="toInteger",
            full_name="tzrpc.proto.toObject.toInteger",
            index=1,
            containing_service=None,
            input_type=Number__pb2._INTEGER,
            output_type=Number__pb2._INTEGER,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="toFloat",
            full_name="tzrpc.proto.toObject.toFloat",
            index=2,
            containing_service=None,
            input_type=Number__pb2._FLOAT,
            output_type=Number__pb2._FLOAT,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="toDouble",
            full_name="tzrpc.proto.toObject.toDouble",
            index=3,
            containing_service=None,
            input_type=Number__pb2._DOUBLE,
            output_type=Number__pb2._DOUBLE,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="toBoolean",
            full_name="tzrpc.proto.toObject.toBoolean",
            index=4,
            containing_service=None,
            input_type=Boolean__pb2._BOOLEAN,
            output_type=Boolean__pb2._BOOLEAN,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="toBytes",
            full_name="tzrpc.proto.toObject.toBytes",
            index=5,
            containing_service=None,
            input_type=Bytes__pb2._BYTES,
            output_type=Bytes__pb2._BYTES,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="toNdarray",
            full_name="tzrpc.proto.toObject.toNdarray",
            index=6,
            containing_service=None,
            input_type=Numpy__pb2._NDARRAY,
            output_type=Numpy__pb2._NDARRAY,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="toTensor",
            full_name="tzrpc.proto.toObject.toTensor",
            index=7,
            containing_service=None,
            input_type=Tensor__pb2._TENSOR,
            output_type=Tensor__pb2._TENSOR,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="toNdarrays",
            full_name="tzrpc.proto.toObject.toNdarrays",
            index=8,
            containing_service=None,
            input_type=Numpy__pb2._NDARRAYS,
            output_type=Numpy__pb2._NDARRAYS,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="toTensors",
            full_name="tzrpc.proto.toObject.toTensors",
            index=9,
            containing_service=None,
            input_type=Tensor__pb2._TENSORS,
            output_type=Tensor__pb2._TENSORS,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="toIntegerArrayList",
            full_name="tzrpc.proto.toObject.toIntegerArrayList",
            index=10,
            containing_service=None,
            input_type=Number__pb2._INTEGERARRAYLIST,
            output_type=Number__pb2._INTEGERARRAYLIST,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="toFloatArrayList",
            full_name="tzrpc.proto.toObject.toFloatArrayList",
            index=11,
            containing_service=None,
            input_type=Number__pb2._FLOATARRAYLIST,
            output_type=Number__pb2._FLOATARRAYLIST,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="toDoubleArrayList",
            full_name="tzrpc.proto.toObject.toDoubleArrayList",
            index=12,
            containing_service=None,
            input_type=Number__pb2._DOUBLEARRAYLIST,
            output_type=Number__pb2._DOUBLEARRAYLIST,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="toBooleanArrayList",
            full_name="tzrpc.proto.toObject.toBooleanArrayList",
            index=13,
            containing_service=None,
            input_type=Boolean__pb2._BOOLEANARRAYLIST,
            output_type=Boolean__pb2._BOOLEANARRAYLIST,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
)
_sym_db.RegisterServiceDescriptor(_TOOBJECT)

DESCRIPTOR.services_by_name["toObject"] = _TOOBJECT

# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: String.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor.FileDescriptor(
    name="String.proto",
    package="tzrpc.proto",
    syntax="proto3",
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n\x0cString.proto\x12\x0btzrpc.proto"\x16\n\x06String\x12\x0c\n\x04text\x18\x01 \x01(\t"\x1f\n\x0fStringArrayList\x12\x0c\n\x04text\x18\x01 \x03(\tb\x06proto3',
)


_STRING = _descriptor.Descriptor(
    name="String",
    full_name="tzrpc.proto.String",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="text",
            full_name="tzrpc.proto.String.text",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=29,
    serialized_end=51,
)


_STRINGARRAYLIST = _descriptor.Descriptor(
    name="StringArrayList",
    full_name="tzrpc.proto.StringArrayList",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="text",
            full_name="tzrpc.proto.StringArrayList.text",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=53,
    serialized_end=84,
)

DESCRIPTOR.message_types_by_name["String"] = _STRING
DESCRIPTOR.message_types_by_name["StringArrayList"] = _STRINGARRAYLIST
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

String = _reflection.GeneratedProtocolMessageType(
    "String",
    (_message.Message,),
    {
        "DESCRIPTOR": _STRING,
        "__module__": "String_pb2"
        # @@protoc_insertion_point(class_scope:tzrpc.proto.String)
    },
)
_sym_db.RegisterMessage(String)

StringArrayList = _reflection.GeneratedProtocolMessageType(
    "StringArrayList",
    (_message.Message,),
    {
        "DESCRIPTOR": _STRINGARRAYLIST,
        "__module__": "String_pb2"
        # @@protoc_insertion_point(class_scope:tzrpc.proto.StringArrayList)
    },
)
_sym_db.RegisterMessage(StringArrayList)


# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: Tensor.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor.FileDescriptor(
    name="Tensor.proto",
    package="tzrpc.proto",
    syntax="proto3",
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n\x0cTensor.proto\x12\x0btzrpc.proto".\n\x07Tensors\x12#\n\x06protos\x18\x01 \x03(\x0b\x32\x13.tzrpc.proto.Tensor"\xe2\x02\n\x06Tensor\x12\x0c\n\x04\x64ims\x18\x01 \x03(\x03\x12/\n\tdata_type\x18\x02 \x01(\x0e\x32\x1c.tzrpc.proto.Tensor.DataType\x12\x12\n\nfloat_data\x18\x03 \x03(\x02\x12\x12\n\nint32_data\x18\x04 \x03(\x05\x12\x11\n\tbyte_data\x18\x05 \x01(\x0c\x12\x13\n\x0bstring_data\x18\x06 \x03(\x0c\x12\x13\n\x0b\x64ouble_data\x18\x07 \x03(\x01\x12\x12\n\nint64_data\x18\x08 \x03(\x03"\x9f\x01\n\x08\x44\x61taType\x12\r\n\tUNDEFINED\x10\x00\x12\t\n\x05\x46LOAT\x10\x01\x12\t\n\x05INT32\x10\x02\x12\x08\n\x04\x42YTE\x10\x03\x12\n\n\x06STRING\x10\x04\x12\x08\n\x04\x42OOL\x10\x05\x12\t\n\x05UINT8\x10\x06\x12\x08\n\x04INT8\x10\x07\x12\n\n\x06UINT16\x10\x08\x12\t\n\x05INT16\x10\t\x12\t\n\x05INT64\x10\n\x12\x0b\n\x07\x46LOAT16\x10\x0c\x12\n\n\x06\x44OUBLE\x10\rb\x06proto3',
)


_TENSOR_DATATYPE = _descriptor.EnumDescriptor(
    name="DataType",
    full_name="tzrpc.proto.Tensor.DataType",
    filename=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
    values=[
        _descriptor.EnumValueDescriptor(
            name="UNDEFINED",
            index=0,
            number=0,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="FLOAT",
            index=1,
            number=1,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="INT32",
            index=2,
            number=2,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="BYTE",
            index=3,
            number=3,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="STRING",
            index=4,
            number=4,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="BOOL",
            index=5,
            number=5,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="UINT8",
            index=6,
            number=6,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="INT8",
            index=7,
            number=7,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="UINT16",
            index=8,
            number=8,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="INT16",
            index=9,
            number=9,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="INT64",
            index=10,
            number=10,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="FLOAT16",
            index=11,
            number=12,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="DOUBLE",
            index=12,
            number=13,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=273,
    serialized_end=432,
)
_sym_db.RegisterEnumDescriptor(_TENSOR_DATATYPE)


_TENSORS = _descriptor.Descriptor(
    name="Tensors",
    full_name="tzrpc.proto.Tensors",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="protos",
            full_name="tzrpc.proto.Tensors.protos",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=29,
    serialized_end=75,
)


_TENSOR = _descriptor.Descriptor(
    name="Tensor",
    full_name="tzrpc.proto.Tensor",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="dims",
            full_name="tzrpc.proto.Tensor.dims",
            index=0,
            number=1,
            type=3,
            cpp_type=2,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="data_type",
            full_name="tzrpc.proto.Tensor.data_type",
            index=1,
            number=2,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="float_data",
            full_name="tzrpc.proto.Tensor.float_data",
            index=2,
            number=3,
            type=2,
            cpp_type=6,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="int32_data",
            full_name="tzrpc.proto.Tensor.int32_data",
            index=3,
            number=4,
            type=5,
            cpp_type=1,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="byte_data",
            full_name="tzrpc.proto.Tensor.byte_data",
            index=4,
            number=5,
            type=12,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"",
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="string_data",
            full_name="tzrpc.proto.Tensor.string_data",
            index=5,
            number=6,
            type=12,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="double_data",
            full_name="tzrpc.proto.Tensor.double_data",
            index=6,
            number=7,
            type=1,
            cpp_type=5,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="int64_data",
            full_name="tzrpc.proto.Tensor.int64_data",
            index=7,
            number=8,
            type=3,
            cpp_type=2,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[
        _TENSOR_DATATYPE,
    ],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=78,
    serialized_end=432,
)

_TENSORS.fields_by_name["protos"].message_type = _TENSOR
_TENSOR.fields_by_name["data_type"].enum_type = _TENSOR_DATATYPE
_TENSOR_DATATYPE.containing_type = _TENSOR
DESCRIPTOR.message_types_by_name["Tensors"] = _TENSORS
DESCRIPTOR.message_types_by_name["Tensor"] = _TENSOR
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Tensors = _reflection.GeneratedProtocolMessageType(
    "Tensors",
    (_message.Message,),
    {
        "DESCRIPTOR": _TENSORS,
        "__module__": "Tensor_pb2"
        # @@protoc_insertion_point(class_scope:tzrpc.proto.Tensors)
    },
)
_sym_db.RegisterMessage(Tensors)

Tensor = _reflection.GeneratedProtocolMessageType(
    "Tensor",
    (_message.Message,),
    {
        "DESCRIPTOR": _TENSOR,
        "__module__": "Tensor_pb2"
        # @@protoc_insertion_point(class_scope:tzrpc.proto.Tensor)
    },
)
_sym_db.RegisterMessage(Tensor)


# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# @Time  : 2021/8/6 12:13
# @Author : lovemefan
# @Email : lovemefan@outlook.com
# @File : base.py
import re
from typing import Any
from warnings import warn

from tzrpc.decorator.rpc import RpcServicer
from tzrpc.exceptions.exceptions import TZRPCException

VALID_NAME = re.compile(r"^[a-zA-Z][a-zA-Z0-9_\-]*$")


class tzrpcBase(RpcServicer):
    def __init__(self, name) -> None:
        super().__init__()
        class_name = self.__class__.__name__

        if name is None:
            raise TZRPCException(
                f"{class_name} instance cannot be unnamed. "
                "Please use TZRPC(name='your_application_name') instead.",
            )

        if not VALID_NAME.match(name):
            warn(
                f"{class_name} instance named '{name}' uses a format that is"
                f"deprecated. Starting in version 21.12, {class_name} objects "
                "must be named only using alphanumeric characters, _, or -.",
                DeprecationWarning,
            )

        self.name = name

        # for base in TZRPCException.__bases__:
        #     base.__init__(self, *args, **kwargs)  # type: ignore

    def __str__(self) -> str:
        return f"<{self.__class__.__name__} {self.name}>"

    def __repr__(self) -> str:
        return f'{self.__class__.__name__}(name="{self.name}")'

    def __setattr__(self, name: str, value: Any) -> None:
        super().__setattr__(name, value)



# -*- coding: utf-8 -*-
# @Time  : 2021/8/17 19:57
# @Author : lovemefan
# @Email : lovemefan@outlook.com
# @File : client.py
import numbers

import numpy as np
import torch

from tzrpc import TZPRC_Client

SERVER_ADDRESS = "localhost:8000"
client = TZPRC_Client(SERVER_ADDRESS)


@client.register
def say_hello2(text):
    return text


@client.register
def send_numpy_obj():
    data = np.random.randn(1, 2, 2, 4)
    return data


@client.register
def send_torch_tensor_obj():
    data = torch.tensor([[1, 2, 3], [4, 5, 6]])
    return data


@client.register
def say_hello(text):
    return text


@client.register
def send_bytes():
    return b"just for test"


@client.register
def send_number(data: numbers.Number):
    return data


@client.register
def send_bool(_bool: bool):
    return _bool


@client.register
def send_python_obj(obj):
    return obj


if __name__ == "__main__":
    print(say_hello(text="lovemefan 1"))
    print(say_hello(text="lovemefan 1"))
    print(say_hello2(text="lovemefan 2"))
    print(send_numpy_obj())
    print(send_torch_tensor_obj())
    print(send_bytes())
    print(send_number(2))
    print(send_number(1 / 3))
    print(send_bool(True))
    print(send_bool(False))

    class testOb:
        def __init__(self, name, age):
            self.name = name
            self.age = age

    python_obj = testOb("test_name", 20)
    print(send_python_obj(python_obj))



# -*- coding: utf-8 -*-
# @Time  : 2021/7/25 21:12
# @Author : lovemefan
# @Email : lovemefan@outlook.com
# @File : exceptions.py
from typing import Optional, Union


class TZRPCException(Exception):
    def __init__(
        self,
        message: Optional[Union[str, bytes]] = "",
    ) -> None:
        super().__init__(message)



# -*- coding: utf-8 -*-
# @Time  : 2021/7/25 13:28
# @Author : lovemefan
# @Email : lovemefan@outlook.com
# @File : BertForVi.py

import torch
from transformers import AutoModel, AutoTokenizer

from examples.bert.BertBase import BertBase
from tzrpc.decorator.singleton import singleton


@singleton
class BertForVietnamese(BertBase):
    def __init__(self):
        # super(self, 'BertForVietnamese')
        self.phobert = AutoModel.from_pretrained("vinai/phobert-base")

        # For transformers v4.x+:
        self.tokenizer = AutoTokenizer.from_pretrained(
            "vinai/phobert-base", use_fast=False
        )

    def output(self, input: str):
        input_ids = torch.tensor([self.tokenizer.encode(input)])

        with torch.no_grad():
            features = self.phobert(input_ids)  # Models outputs are now tuples
        return features


if __name__ == "__main__":
    line = "Tôi là sinh_viên trường đại_học Công_nghệ ."
    vi = BertForVietnamese()
    out = vi.output(line)
    123



#!/usr/bin/python3
# -*- coding: utf-8 -*-
# @Time    : 2020/12/24 下午2:32
# @Author  : lovemefan
# @File    : singleton.py
import logging
import threading

logger = logging.getLogger(__name__)

lock = threading.Lock()
# instance container
instances = {}


def singleton(cls):
    """this is decorator to decorate class , make the class singleton(修饰器实现单例模式)"""

    def get_instance(*args, **kwargs):
        cls_name = cls.__name__
        try:
            lock.acquire()
            if cls_name not in instances:
                logger.info(f"creating {cls_name} instance")
                instance = cls(*args, **kwargs)
                instances[cls_name] = instance
                logger.info(f"create {cls_name} instance finished")
        finally:
            lock.release()

        return instances[cls_name]

    return get_instance


def get_all_instance():
    """return all instance in the container"""
    return instances



# -*- coding:utf-8 -*-
# @FileName  :constant.py
# @Time      :2023/7/25 17:29
# @Author    :lovemefan
# @Email     :lovemefan@outlook.com
from tzrpc.proto.py.Boolean_pb2 import Boolean
from tzrpc.proto.py.Bytes_pb2 import Bytes
from tzrpc.proto.py.Numpy_pb2 import ndarray
from tzrpc.proto.py.String_pb2 import String
from tzrpc.proto.py.Tensor_pb2 import Tensor

SUPPORT_TYPE = {
    "String": String,
    "Boolean": Boolean,
    "Bytes": Bytes,
    "Ndarray": ndarray,
    "Tensor": Tensor,
}

MAX_MESSAGE_LENGTH = 500*1024*1024

MAX_METADATA_SIZE = 1024*16



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from tzrpc.proto.py import Boolean_pb2 as Boolean__pb2
from tzrpc.proto.py import Bytes_pb2 as Bytes__pb2
from tzrpc.proto.py import Number_pb2 as Number__pb2
from tzrpc.proto.py import Numpy_pb2 as Numpy__pb2
from tzrpc.proto.py import String_pb2 as String__pb2
from tzrpc.proto.py import Tensor_pb2 as Tensor__pb2


class toObjectStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel, suffix=None):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        suffix = suffix or ""
        self.toString = channel.unary_unary(
            f"/tzrpc.proto.toObject/toString-{suffix}",
            request_serializer=String__pb2.String.SerializeToString,
            response_deserializer=String__pb2.String.FromString,
        )
        self.toInteger = channel.unary_unary(
            f"/tzrpc.proto.toObject/toInteger-{suffix}",
            request_serializer=Number__pb2.Integer.SerializeToString,
            response_deserializer=Number__pb2.Integer.FromString,
        )
        self.toFloat = channel.unary_unary(
            f"/tzrpc.proto.toObject/toFloat-{suffix}",
            request_serializer=Number__pb2.Float.SerializeToString,
            response_deserializer=Number__pb2.Float.FromString,
        )
        self.toDouble = channel.unary_unary(
            f"/tzrpc.proto.toObject/toDouble-{suffix}",
            request_serializer=Number__pb2.Double.SerializeToString,
            response_deserializer=Number__pb2.Double.FromString,
        )
        self.toBoolean = channel.unary_unary(
            f"/tzrpc.proto.toObject/toBoolean-{suffix}",
            request_serializer=Boolean__pb2.Boolean.SerializeToString,
            response_deserializer=Boolean__pb2.Boolean.FromString,
        )
        self.toBytes = channel.unary_unary(
            f"/tzrpc.proto.toObject/toBytes-{suffix}",
            request_serializer=Bytes__pb2.Bytes.SerializeToString,
            response_deserializer=Bytes__pb2.Bytes.FromString,
        )
        self.toNdarray = channel.unary_unary(
            f"/tzrpc.proto.toObject/toNdarray-{suffix}",
            request_serializer=Numpy__pb2.ndarray.SerializeToString,
            response_deserializer=Numpy__pb2.ndarray.FromString,
        )
        self.toTensor = channel.unary_unary(
            f"/tzrpc.proto.toObject/toTensor-{suffix}",
            request_serializer=Tensor__pb2.Tensor.SerializeToString,
            response_deserializer=Tensor__pb2.Tensor.FromString,
        )
        self.toNdarrays = channel.unary_unary(
            f"/tzrpc.proto.toObject/toNdarrays-{suffix}",
            request_serializer=Numpy__pb2.ndarrays.SerializeToString,
            response_deserializer=Numpy__pb2.ndarrays.FromString,
        )
        self.toTensors = channel.unary_unary(
            f"/tzrpc.proto.toObject/toTensors-{suffix}",
            request_serializer=Tensor__pb2.Tensors.SerializeToString,
            response_deserializer=Tensor__pb2.Tensors.FromString,
        )
        self.toIntegerArrayList = channel.unary_unary(
            f"/tzrpc.proto.toObject/toIntegerArrayList-{suffix}",
            request_serializer=Number__pb2.IntegerArrayList.SerializeToString,
            response_deserializer=Number__pb2.IntegerArrayList.FromString,
        )
        self.toFloatArrayList = channel.unary_unary(
            f"/tzrpc.proto.toObject/toFloatArrayList-{suffix}",
            request_serializer=Number__pb2.FloatArrayList.SerializeToString,
            response_deserializer=Number__pb2.FloatArrayList.FromString,
        )
        self.toDoubleArrayList = channel.unary_unary(
            f"/tzrpc.proto.toObject/toDoubleArrayList-{suffix}",
            request_serializer=Number__pb2.DoubleArrayList.SerializeToString,
            response_deserializer=Number__pb2.DoubleArrayList.FromString,
        )
        self.toBooleanArrayList = channel.unary_unary(
            f"/tzrpc.proto.toObject/toBooleanArrayList-{suffix}",
            request_serializer=Boolean__pb2.BooleanArrayList.SerializeToString,
            response_deserializer=Boolean__pb2.BooleanArrayList.FromString,
        )


class toObjectServicer(object):
    """Missing associated documentation comment in .proto file."""

    def toString(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def toInteger(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def toFloat(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def toDouble(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def toBoolean(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def toBytes(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def toNdarray(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def toTensor(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def toNdarrays(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def toTensors(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def toIntegerArrayList(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def toFloatArrayList(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def toDoubleArrayList(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def toBooleanArrayList(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_toObjectServicer_to_server(servicer, server, suffix=None):
    _tmp_rpc_method_handlers = {
        "toString": grpc.unary_unary_rpc_method_handler(
            servicer.toString,
            request_deserializer=String__pb2.String.FromString,
            response_serializer=String__pb2.String.SerializeToString,
        ),
        "toInteger": grpc.unary_unary_rpc_method_handler(
            servicer.toInteger,
            request_deserializer=Number__pb2.Integer.FromString,
            response_serializer=Number__pb2.Integer.SerializeToString,
        ),
        "toFloat": grpc.unary_unary_rpc_method_handler(
            servicer.toFloat,
            request_deserializer=Number__pb2.Float.FromString,
            response_serializer=Number__pb2.Float.SerializeToString,
        ),
        "toDouble": grpc.unary_unary_rpc_method_handler(
            servicer.toDouble,
            request_deserializer=Number__pb2.Double.FromString,
            response_serializer=Number__pb2.Double.SerializeToString,
        ),
        "toBoolean": grpc.unary_unary_rpc_method_handler(
            servicer.toBoolean,
            request_deserializer=Boolean__pb2.Boolean.FromString,
            response_serializer=Boolean__pb2.Boolean.SerializeToString,
        ),
        "toBytes": grpc.unary_unary_rpc_method_handler(
            servicer.toBytes,
            request_deserializer=Bytes__pb2.Bytes.FromString,
            response_serializer=Bytes__pb2.Bytes.SerializeToString,
        ),
        "toNdarray": grpc.unary_unary_rpc_method_handler(
            servicer.toNdarray,
            request_deserializer=Numpy__pb2.ndarray.FromString,
            response_serializer=Numpy__pb2.ndarray.SerializeToString,
        ),
        "toTensor": grpc.unary_unary_rpc_method_handler(
            servicer.toTensor,
            request_deserializer=Tensor__pb2.Tensor.FromString,
            response_serializer=Tensor__pb2.Tensor.SerializeToString,
        ),
        "toNdarrays": grpc.unary_unary_rpc_method_handler(
            servicer.toNdarrays,
            request_deserializer=Numpy__pb2.ndarrays.FromString,
            response_serializer=Numpy__pb2.ndarrays.SerializeToString,
        ),
        "toTensors": grpc.unary_unary_rpc_method_handler(
            servicer.toTensors,
            request_deserializer=Tensor__pb2.Tensors.FromString,
            response_serializer=Tensor__pb2.Tensors.SerializeToString,
        ),
        "toIntegerArrayList": grpc.unary_unary_rpc_method_handler(
            servicer.toIntegerArrayList,
            request_deserializer=Number__pb2.IntegerArrayList.FromString,
            response_serializer=Number__pb2.IntegerArrayList.SerializeToString,
        ),
        "toFloatArrayList": grpc.unary_unary_rpc_method_handler(
            servicer.toFloatArrayList,
            request_deserializer=Number__pb2.FloatArrayList.FromString,
            response_serializer=Number__pb2.FloatArrayList.SerializeToString,
        ),
        "toDoubleArrayList": grpc.unary_unary_rpc_method_handler(
            servicer.toDoubleArrayList,
            request_deserializer=Number__pb2.DoubleArrayList.FromString,
            response_serializer=Number__pb2.DoubleArrayList.SerializeToString,
        ),
        "toBooleanArrayList": grpc.unary_unary_rpc_method_handler(
            servicer.toBooleanArrayList,
            request_deserializer=Boolean__pb2.BooleanArrayList.FromString,
            response_serializer=Boolean__pb2.BooleanArrayList.SerializeToString,
        ),
    }

    rpc_method_handlers = dict()
    for key, value in _tmp_rpc_method_handlers.items():
        key = key if suffix is None else f"{key}-{suffix}"
        rpc_method_handlers[key] = value

    generic_handler = grpc.method_handlers_generic_handler(
        "tzrpc.proto.toObject", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class toObject(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def toString(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/tzrpc.proto.toObject/toString",
            String__pb2.String.SerializeToString,
            String__pb2.String.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def toInteger(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/tzrpc.proto.toObject/toInteger",
            Number__pb2.Integer.SerializeToString,
            Number__pb2.Integer.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def toFloat(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/tzrpc.proto.toObject/toFloat",
            Number__pb2.Float.SerializeToString,
            Number__pb2.Float.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def toDouble(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/tzrpc.proto.toObject/toDouble",
            Number__pb2.Double.SerializeToString,
            Number__pb2.Double.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def toBoolean(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/tzrpc.proto.toObject/toBoolean",
            Boolean__pb2.Boolean.SerializeToString,
            Boolean__pb2.Boolean.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def toBytes(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/tzrpc.proto.toObject/toBytes",
            Bytes__pb2.Bytes.SerializeToString,
            Bytes__pb2.Bytes.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def toNdarray(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/tzrpc.proto.toObject/toNdarray",
            Numpy__pb2.ndarray.SerializeToString,
            Numpy__pb2.ndarray.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def toTensor(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/tzrpc.proto.toObject/toTensor",
            Tensor__pb2.Tensor.SerializeToString,
            Tensor__pb2.Tensor.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def toNdarrays(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/tzrpc.proto.toObject/toNdarrays",
            Numpy__pb2.ndarrays.SerializeToString,
            Numpy__pb2.ndarrays.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def toTensors(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/tzrpc.proto.toObject/toTensors",
            Tensor__pb2.Tensors.SerializeToString,
            Tensor__pb2.Tensors.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def toIntegerArrayList(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/tzrpc.proto.toObject/toIntegerArrayList",
            Number__pb2.IntegerArrayList.SerializeToString,
            Number__pb2.IntegerArrayList.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def toFloatArrayList(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/tzrpc.proto.toObject/toFloatArrayList",
            Number__pb2.FloatArrayList.SerializeToString,
            Number__pb2.FloatArrayList.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def toDoubleArrayList(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/tzrpc.proto.toObject/toDoubleArrayList",
            Number__pb2.DoubleArrayList.SerializeToString,
            Number__pb2.DoubleArrayList.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def toBooleanArrayList(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/tzrpc.proto.toObject/toBooleanArrayList",
            Boolean__pb2.BooleanArrayList.SerializeToString,
            Boolean__pb2.BooleanArrayList.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )



# -*- coding: utf-8 -*-
# @Time  : 2021/8/17 21:41
# @Author : lovemefan
# @Email : lovemefan@outlook.com
# @File : client.py
import logging
import numbers
import pickle

import grpc
import numpy as np

from tzrpc.proto.py.Boolean_pb2 import Boolean
from tzrpc.proto.py.Bytes_pb2 import Bytes
from tzrpc.proto.py.Number_pb2 import Double, Integer
from tzrpc.proto.py.Server_pb2_grpc import toObjectStub
from tzrpc.proto.py.String_pb2 import String
from tzrpc.utils.constant import MAX_MESSAGE_LENGTH, MAX_METADATA_SIZE
from tzrpc.utils.logger import get_logger
from tzrpc.utils.numpy_serialized import numpy2protobuf, protobuf2numpy

logger = get_logger(to_std=True, stdout_level="INFO", save_log_file=False)

try:
    import torch
except ImportError:
    print(
        "if you want use torch, please install torch with command `pip install torch` "
    )


class TZPRC_Client:
    __type = [str, np.ndarray, bytes, numbers.Number, bool]
    tensor_type = None
    try:
        import torch

        tensor_type = torch.Tensor
        __type.append(torch.Tensor)
    except ImportError:
        logger.info(
            """
        Please pip install torch to get tensor support
        """
        )

    def __init__(self, server_address: str, debug=False):
        if debug:
            logger.setLevel(logging.DEBUG)
        self.server_address = server_address
        self.channel = grpc.insecure_channel(server_address,
                                             options=[('grpc.max_send_message_length', MAX_MESSAGE_LENGTH),
                                                      ('grpc.max_receive_message_length', MAX_MESSAGE_LENGTH),
                                                      ('grpc.max_metadata_size', MAX_METADATA_SIZE)]
                                             )

    def register(self, func):
        """
        :param return_type: type of return [String, ]
        :return:
        """
        # if return_type not in self.__type:
        #     raise ValueError(f"TZRPC return type only support {self.__type}")

        def wrapper(*args, **kwargs):
            stub = toObjectStub(self.channel, func.__name__)
            result = func(*args, **kwargs)
            if isinstance(result, str):
                request = String(text=result)
                response = stub.toString(request).text

            elif isinstance(result, np.ndarray):
                request = numpy2protobuf(result)
                response = protobuf2numpy(stub.toNdarray(request))
            elif isinstance(result, bytes):
                request = Bytes(data=result)
                response = stub.toBytes(request).data
            elif isinstance(result, bool):
                request = Boolean(value=result)
                response = bool(stub.toBoolean(request).value)
            elif isinstance(result, numbers.Number):
                if isinstance(result, int):
                    request = Integer(value=result)
                    response = stub.toInteger(request).value
                elif isinstance(result, float):
                    request = Double(value=result)
                    response = stub.toDouble(request).value
                else:
                    logger.exception(f"Type of {type(result)} is not support")
            elif self.tensor_type is not None and isinstance(result, self.tensor_type):
                request = numpy2protobuf(result.numpy())
                response = torch.from_numpy(
                    protobuf2numpy(stub.toNdarray(request)).copy()
                )
            else:
                logger.info(f"Data will serialized by pickle and send with bytes")
                obj = pickle.dumps(result)
                request = Bytes(data=b"PICKLE"+obj)
                response = pickle.loads(stub.toBytes(request).data)
                logger.debug(f"type of data loaded is {type(response)}")

            return response

        return wrapper



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: Bytes.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor.FileDescriptor(
    name="Bytes.proto",
    package="tzrpc.proto",
    syntax="proto3",
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n\x0b\x42ytes.proto\x12\x0btzrpc.proto"\x15\n\x05\x42ytes\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\x62\x06proto3',
)


_BYTES = _descriptor.Descriptor(
    name="Bytes",
    full_name="tzrpc.proto.Bytes",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="data",
            full_name="tzrpc.proto.Bytes.data",
            index=0,
            number=1,
            type=12,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"",
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=28,
    serialized_end=49,
)

DESCRIPTOR.message_types_by_name["Bytes"] = _BYTES
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Bytes = _reflection.GeneratedProtocolMessageType(
    "Bytes",
    (_message.Message,),
    {
        "DESCRIPTOR": _BYTES,
        "__module__": "Bytes_pb2"
        # @@protoc_insertion_point(class_scope:tzrpc.proto.Bytes)
    },
)
_sym_db.RegisterMessage(Bytes)


# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# @Time  : 2021/8/17 19:37
# @Author : lovemefan
# @Email : lovemefan@outlook.com
# @File : server.py
import numbers

from tzrpc import TZRPC_Server

server = TZRPC_Server(__name__, debug=False)


@server.register
def send_numpy_obj(data):
    return data * 2 + 1


@server.register
def send_torch_tensor_obj(data):
    return data @ data.T


@server.register
def say_hello(text: str):
    return "hello world " + text


@server.register
def say_hello2(text: str):
    return "hello world 3 " + text


@server.register
def send_bytes(data: bytes):
    return data + data


@server.register
def send_number(data: numbers.Number):
    return data * 2


@server.register
def send_bool(_bool: bool):
    return not _bool


@server.register
def send_python_obj(data):
    print(data)
    return data


if __name__ == "__main__":
    server.run("localhost", 8000)




# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: Boolean.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor.FileDescriptor(
    name="Boolean.proto",
    package="tzrpc.proto",
    syntax="proto3",
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n\rBoolean.proto\x12\x0btzrpc.proto"\x18\n\x07\x42oolean\x12\r\n\x05value\x18\x01 \x01(\x08"!\n\x10\x42ooleanArrayList\x12\r\n\x05value\x18\x01 \x03(\x08\x62\x06proto3',
)


_BOOLEAN = _descriptor.Descriptor(
    name="Boolean",
    full_name="tzrpc.proto.Boolean",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="value",
            full_name="tzrpc.proto.Boolean.value",
            index=0,
            number=1,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=30,
    serialized_end=54,
)


_BOOLEANARRAYLIST = _descriptor.Descriptor(
    name="BooleanArrayList",
    full_name="tzrpc.proto.BooleanArrayList",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="value",
            full_name="tzrpc.proto.BooleanArrayList.value",
            index=0,
            number=1,
            type=8,
            cpp_type=7,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=56,
    serialized_end=89,
)

DESCRIPTOR.message_types_by_name["Boolean"] = _BOOLEAN
DESCRIPTOR.message_types_by_name["BooleanArrayList"] = _BOOLEANARRAYLIST
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Boolean = _reflection.GeneratedProtocolMessageType(
    "Boolean",
    (_message.Message,),
    {
        "DESCRIPTOR": _BOOLEAN,
        "__module__": "Boolean_pb2"
        # @@protoc_insertion_point(class_scope:tzrpc.proto.Boolean)
    },
)
_sym_db.RegisterMessage(Boolean)

BooleanArrayList = _reflection.GeneratedProtocolMessageType(
    "BooleanArrayList",
    (_message.Message,),
    {
        "DESCRIPTOR": _BOOLEANARRAYLIST,
        "__module__": "Boolean_pb2"
        # @@protoc_insertion_point(class_scope:tzrpc.proto.BooleanArrayList)
    },
)
_sym_db.RegisterMessage(BooleanArrayList)


# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: Numpy.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor.FileDescriptor(
    name="Numpy.proto",
    package="tzrpc.proto",
    syntax="proto3",
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n\x0bNumpy.proto\x12\x0btzrpc.proto"\xa8\x04\n\x05\x64type\x12%\n\x04type\x18\x01 \x01(\x0e\x32\x17.tzrpc.proto.dtype.Type\x12\x30\n\nbyte_order\x18\x65 \x01(\x0e\x32\x1c.tzrpc.proto.dtype.ByteOrder\x12\x0e\n\x05names\x18\xc9\x01 \x03(\t\x12/\n\x06\x66ields\x18\xad\x02 \x03(\x0b\x32\x1e.tzrpc.proto.dtype.FieldsEntry\x12(\n\x08subdtype\x18\xe9\x07 \x01(\x0b\x32\x15.tzrpc.proto.subdtype\x1a\x41\n\x0b\x46ieldsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12!\n\x05value\x18\x02 \x01(\x0b\x32\x12.tzrpc.proto.dtype:\x02\x38\x01"B\n\tByteOrder\x12\x11\n\rLITTLE_ENDIAN\x10\x00\x12\x0e\n\nBIG_ENDIAN\x10\x01\x12\n\n\x06NATIVE\x10\x02\x12\x06\n\x02NA\x10\x03"\xd3\x01\n\x04Type\x12\x0b\n\x07\x66loat64\x10\x00\x12\x0b\n\x07\x66loat32\x10\x01\x12\x0b\n\x07\x66loat16\x10\x02\x12\x0e\n\ncomplex128\x10\x03\x12\r\n\tcomplex64\x10\x04\x12\n\n\x06uint64\x10\x05\x12\n\n\x06uint32\x10\x06\x12\n\n\x06uint16\x10\x07\x12\t\n\x05uint8\x10\x08\x12\t\n\x05int64\x10\t\x12\t\n\x05int32\x10\n\x12\t\n\x05int16\x10\x0b\x12\x08\n\x04int8\x10\x0c\x12\x08\n\x04S128\x10\r\x12\x07\n\x03S64\x10\x0e\x12\x07\n\x03S32\x10\x0f\x12\x07\n\x03S16\x10\x10\x12\x06\n\x02S8\x10\x11"@\n\x08subdtype\x12%\n\x04type\x18\x01 \x01(\x0e\x32\x17.tzrpc.proto.dtype.Type\x12\r\n\x05shape\x18\x65 \x03(\x03"\\\n\x07ndarray\x12\r\n\x05shape\x18\x01 \x03(\x03\x12!\n\x05\x64type\x18\x65 \x01(\x0b\x32\x12.tzrpc.proto.dtype\x12\r\n\x04\x64\x61ta\x18\xc9\x01 \x01(\x0c\x12\x10\n\x07strides\x18\xad\x02 \x03(\x03"1\n\x08ndarrays\x12%\n\x07ndarray\x18\x01 \x03(\x0b\x32\x14.tzrpc.proto.ndarrayb\x06proto3',
)


_DTYPE_BYTEORDER = _descriptor.EnumDescriptor(
    name="ByteOrder",
    full_name="tzrpc.proto.dtype.ByteOrder",
    filename=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
    values=[
        _descriptor.EnumValueDescriptor(
            name="LITTLE_ENDIAN",
            index=0,
            number=0,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="BIG_ENDIAN",
            index=1,
            number=1,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="NATIVE",
            index=2,
            number=2,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="NA",
            index=3,
            number=3,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=301,
    serialized_end=367,
)
_sym_db.RegisterEnumDescriptor(_DTYPE_BYTEORDER)

_DTYPE_TYPE = _descriptor.EnumDescriptor(
    name="Type",
    full_name="tzrpc.proto.dtype.Type",
    filename=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
    values=[
        _descriptor.EnumValueDescriptor(
            name="float64",
            index=0,
            number=0,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="float32",
            index=1,
            number=1,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="float16",
            index=2,
            number=2,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="complex128",
            index=3,
            number=3,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="complex64",
            index=4,
            number=4,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="uint64",
            index=5,
            number=5,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="uint32",
            index=6,
            number=6,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="uint16",
            index=7,
            number=7,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="uint8",
            index=8,
            number=8,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="int64",
            index=9,
            number=9,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="int32",
            index=10,
            number=10,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="int16",
            index=11,
            number=11,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="int8",
            index=12,
            number=12,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="S128",
            index=13,
            number=13,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="S64",
            index=14,
            number=14,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="S32",
            index=15,
            number=15,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="S16",
            index=16,
            number=16,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="S8",
            index=17,
            number=17,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=370,
    serialized_end=581,
)
_sym_db.RegisterEnumDescriptor(_DTYPE_TYPE)


_DTYPE_FIELDSENTRY = _descriptor.Descriptor(
    name="FieldsEntry",
    full_name="tzrpc.proto.dtype.FieldsEntry",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="tzrpc.proto.dtype.FieldsEntry.key",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="value",
            full_name="tzrpc.proto.dtype.FieldsEntry.value",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=b"8\001",
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=234,
    serialized_end=299,
)

_DTYPE = _descriptor.Descriptor(
    name="dtype",
    full_name="tzrpc.proto.dtype",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="type",
            full_name="tzrpc.proto.dtype.type",
            index=0,
            number=1,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="byte_order",
            full_name="tzrpc.proto.dtype.byte_order",
            index=1,
            number=101,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="names",
            full_name="tzrpc.proto.dtype.names",
            index=2,
            number=201,
            type=9,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="fields",
            full_name="tzrpc.proto.dtype.fields",
            index=3,
            number=301,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="subdtype",
            full_name="tzrpc.proto.dtype.subdtype",
            index=4,
            number=1001,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[
        _DTYPE_FIELDSENTRY,
    ],
    enum_types=[
        _DTYPE_BYTEORDER,
        _DTYPE_TYPE,
    ],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=29,
    serialized_end=581,
)


_SUBDTYPE = _descriptor.Descriptor(
    name="subdtype",
    full_name="tzrpc.proto.subdtype",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="type",
            full_name="tzrpc.proto.subdtype.type",
            index=0,
            number=1,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="shape",
            full_name="tzrpc.proto.subdtype.shape",
            index=1,
            number=101,
            type=3,
            cpp_type=2,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=583,
    serialized_end=647,
)


_NDARRAY = _descriptor.Descriptor(
    name="ndarray",
    full_name="tzrpc.proto.ndarray",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="shape",
            full_name="tzrpc.proto.ndarray.shape",
            index=0,
            number=1,
            type=3,
            cpp_type=2,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="dtype",
            full_name="tzrpc.proto.ndarray.dtype",
            index=1,
            number=101,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="data",
            full_name="tzrpc.proto.ndarray.data",
            index=2,
            number=201,
            type=12,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"",
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="strides",
            full_name="tzrpc.proto.ndarray.strides",
            index=3,
            number=301,
            type=3,
            cpp_type=2,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=649,
    serialized_end=741,
)


_NDARRAYS = _descriptor.Descriptor(
    name="ndarrays",
    full_name="tzrpc.proto.ndarrays",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="ndarray",
            full_name="tzrpc.proto.ndarrays.ndarray",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=743,
    serialized_end=792,
)

_DTYPE_FIELDSENTRY.fields_by_name["value"].message_type = _DTYPE
_DTYPE_FIELDSENTRY.containing_type = _DTYPE
_DTYPE.fields_by_name["type"].enum_type = _DTYPE_TYPE
_DTYPE.fields_by_name["byte_order"].enum_type = _DTYPE_BYTEORDER
_DTYPE.fields_by_name["fields"].message_type = _DTYPE_FIELDSENTRY
_DTYPE.fields_by_name["subdtype"].message_type = _SUBDTYPE
_DTYPE_BYTEORDER.containing_type = _DTYPE
_DTYPE_TYPE.containing_type = _DTYPE
_SUBDTYPE.fields_by_name["type"].enum_type = _DTYPE_TYPE
_NDARRAY.fields_by_name["dtype"].message_type = _DTYPE
_NDARRAYS.fields_by_name["ndarray"].message_type = _NDARRAY
DESCRIPTOR.message_types_by_name["dtype"] = _DTYPE
DESCRIPTOR.message_types_by_name["subdtype"] = _SUBDTYPE
DESCRIPTOR.message_types_by_name["ndarray"] = _NDARRAY
DESCRIPTOR.message_types_by_name["ndarrays"] = _NDARRAYS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

dtype = _reflection.GeneratedProtocolMessageType(
    "dtype",
    (_message.Message,),
    {
        "FieldsEntry": _reflection.GeneratedProtocolMessageType(
            "FieldsEntry",
            (_message.Message,),
            {
                "DESCRIPTOR": _DTYPE_FIELDSENTRY,
                "__module__": "Numpy_pb2"
                # @@protoc_insertion_point(class_scope:tzrpc.proto.dtype.FieldsEntry)
            },
        ),
        "DESCRIPTOR": _DTYPE,
        "__module__": "Numpy_pb2"
        # @@protoc_insertion_point(class_scope:tzrpc.proto.dtype)
    },
)
_sym_db.RegisterMessage(dtype)
_sym_db.RegisterMessage(dtype.FieldsEntry)

subdtype = _reflection.GeneratedProtocolMessageType(
    "subdtype",
    (_message.Message,),
    {
        "DESCRIPTOR": _SUBDTYPE,
        "__module__": "Numpy_pb2"
        # @@protoc_insertion_point(class_scope:tzrpc.proto.subdtype)
    },
)
_sym_db.RegisterMessage(subdtype)

ndarray = _reflection.GeneratedProtocolMessageType(
    "ndarray",
    (_message.Message,),
    {
        "DESCRIPTOR": _NDARRAY,
        "__module__": "Numpy_pb2"
        # @@protoc_insertion_point(class_scope:tzrpc.proto.ndarray)
    },
)
_sym_db.RegisterMessage(ndarray)

ndarrays = _reflection.GeneratedProtocolMessageType(
    "ndarrays",
    (_message.Message,),
    {
        "DESCRIPTOR": _NDARRAYS,
        "__module__": "Numpy_pb2"
        # @@protoc_insertion_point(class_scope:tzrpc.proto.ndarrays)
    },
)
_sym_db.RegisterMessage(ndarrays)


_DTYPE_FIELDSENTRY._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding:utf-8 -*-
# @FileName  :convert2numpy.py
# @Time      :2023/7/25 22:38
# @Author    :lovemefan
# @Email     :lovemefan@outlook.com
import numpy as np

from tzrpc.proto.py.Numpy_pb2 import dtype, ndarray

BYTE_ORDER_DICT = {
    "<": dtype.ByteOrder.LITTLE_ENDIAN,
    ">": dtype.ByteOrder.BIG_ENDIAN,
    "=": dtype.ByteOrder.NATIVE,
    "|": dtype.ByteOrder.NA,
}

BYTE_ORDER_DICT_REVERSE = {value: key for key, value in BYTE_ORDER_DICT.items()}

DATA_TYPE_DICT = {
    "float64": dtype.Type.float64,
    "float32": dtype.Type.float32,
    "float16": dtype.Type.float16,
    "complex128": dtype.Type.complex128,
    "complex64": dtype.Type.complex64,
    "uint64": dtype.Type.uint64,
    "uint32": dtype.Type.uint32,
    "uint16": dtype.Type.uint16,
    "uint8": dtype.Type.uint8,
    "int64": dtype.Type.int64,
    "int32": dtype.Type.int32,
    "int16": dtype.Type.int16,
    "int8": dtype.Type.int8,
    "S128": dtype.Type.S128,
    "S64": dtype.Type.S64,
    "S32": dtype.Type.S32,
    "S16": dtype.Type.S16,
    "S8": dtype.Type.S8,
}

DATA_TYPE_DICT_REVERSE = {value: key for key, value in DATA_TYPE_DICT.items()}


def numpy2protobuf(data: np.ndarray):
    def get_byteorder(_data: np.ndarray):
        order = _data.dtype.byteorder
        if order in BYTE_ORDER_DICT:
            return BYTE_ORDER_DICT[order]
        else:
            raise ValueError(f"byte order: {order} not support ")

    def get_ndarray_type(_data: np.ndarray):
        _type = _data.dtype.name
        if _type in DATA_TYPE_DICT:
            return DATA_TYPE_DICT[_type]
        else:
            raise ValueError(f"data type: {_type} not support ")

    buf = ndarray(
        shape=data.shape,
        dtype=dtype(byte_order=get_byteorder(data), type=get_ndarray_type(data)),
        data=data.data.tobytes(),
        strides=data.strides,
    )
    return buf


def protobuf2numpy(buf: ndarray):
    def get_ndarray_type(_data: ndarray):
        _type = _data.dtype.type
        if _type in DATA_TYPE_DICT_REVERSE:
            return DATA_TYPE_DICT_REVERSE[_type]
        else:
            raise ValueError(f"data type: {_type} not support ")

    return np.frombuffer(buf.data, dtype=get_ndarray_type(buf)).reshape(
        tuple(buf.shape)
    )


if __name__ == "__main__":
    import numpy as np

    np_obj = np.random.randn(2, 3, 4).view(np.int64)
    print(np_obj)
    a = numpy2protobuf(np_obj)
    print(a)
    b = protobuf2numpy(a)
    print(b)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: Number.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor.FileDescriptor(
    name="Number.proto",
    package="tzrpc.proto",
    syntax="proto3",
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n\x0cNumber.proto\x12\x0btzrpc.proto"\x18\n\x07Integer\x12\r\n\x05value\x18\x01 \x01(\x03"\x16\n\x05\x46loat\x12\r\n\x05value\x18\x01 \x01(\x02"\x17\n\x06\x44ouble\x12\r\n\x05value\x18\x01 \x01(\x01"!\n\x10IntegerArrayList\x12\r\n\x05value\x18\x01 \x03(\x05"\x1f\n\x0e\x46loatArrayList\x12\r\n\x05value\x18\x01 \x03(\x02" \n\x0f\x44oubleArrayList\x12\r\n\x05value\x18\x01 \x03(\x01\x62\x06proto3',
)


_INTEGER = _descriptor.Descriptor(
    name="Integer",
    full_name="tzrpc.proto.Integer",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="value",
            full_name="tzrpc.proto.Integer.value",
            index=0,
            number=1,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=29,
    serialized_end=53,
)


_FLOAT = _descriptor.Descriptor(
    name="Float",
    full_name="tzrpc.proto.Float",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="value",
            full_name="tzrpc.proto.Float.value",
            index=0,
            number=1,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=55,
    serialized_end=77,
)


_DOUBLE = _descriptor.Descriptor(
    name="Double",
    full_name="tzrpc.proto.Double",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="value",
            full_name="tzrpc.proto.Double.value",
            index=0,
            number=1,
            type=1,
            cpp_type=5,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=79,
    serialized_end=102,
)


_INTEGERARRAYLIST = _descriptor.Descriptor(
    name="IntegerArrayList",
    full_name="tzrpc.proto.IntegerArrayList",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="value",
            full_name="tzrpc.proto.IntegerArrayList.value",
            index=0,
            number=1,
            type=5,
            cpp_type=1,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=104,
    serialized_end=137,
)


_FLOATARRAYLIST = _descriptor.Descriptor(
    name="FloatArrayList",
    full_name="tzrpc.proto.FloatArrayList",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="value",
            full_name="tzrpc.proto.FloatArrayList.value",
            index=0,
            number=1,
            type=2,
            cpp_type=6,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=139,
    serialized_end=170,
)


_DOUBLEARRAYLIST = _descriptor.Descriptor(
    name="DoubleArrayList",
    full_name="tzrpc.proto.DoubleArrayList",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="value",
            full_name="tzrpc.proto.DoubleArrayList.value",
            index=0,
            number=1,
            type=1,
            cpp_type=5,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=172,
    serialized_end=204,
)

DESCRIPTOR.message_types_by_name["Integer"] = _INTEGER
DESCRIPTOR.message_types_by_name["Float"] = _FLOAT
DESCRIPTOR.message_types_by_name["Double"] = _DOUBLE
DESCRIPTOR.message_types_by_name["IntegerArrayList"] = _INTEGERARRAYLIST
DESCRIPTOR.message_types_by_name["FloatArrayList"] = _FLOATARRAYLIST
DESCRIPTOR.message_types_by_name["DoubleArrayList"] = _DOUBLEARRAYLIST
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Integer = _reflection.GeneratedProtocolMessageType(
    "Integer",
    (_message.Message,),
    {
        "DESCRIPTOR": _INTEGER,
        "__module__": "Number_pb2"
        # @@protoc_insertion_point(class_scope:tzrpc.proto.Integer)
    },
)
_sym_db.RegisterMessage(Integer)

Float = _reflection.GeneratedProtocolMessageType(
    "Float",
    (_message.Message,),
    {
        "DESCRIPTOR": _FLOAT,
        "__module__": "Number_pb2"
        # @@protoc_insertion_point(class_scope:tzrpc.proto.Float)
    },
)
_sym_db.RegisterMessage(Float)

Double = _reflection.GeneratedProtocolMessageType(
    "Double",
    (_message.Message,),
    {
        "DESCRIPTOR": _DOUBLE,
        "__module__": "Number_pb2"
        # @@protoc_insertion_point(class_scope:tzrpc.proto.Double)
    },
)
_sym_db.RegisterMessage(Double)

IntegerArrayList = _reflection.GeneratedProtocolMessageType(
    "IntegerArrayList",
    (_message.Message,),
    {
        "DESCRIPTOR": _INTEGERARRAYLIST,
        "__module__": "Number_pb2"
        # @@protoc_insertion_point(class_scope:tzrpc.proto.IntegerArrayList)
    },
)
_sym_db.RegisterMessage(IntegerArrayList)

FloatArrayList = _reflection.GeneratedProtocolMessageType(
    "FloatArrayList",
    (_message.Message,),
    {
        "DESCRIPTOR": _FLOATARRAYLIST,
        "__module__": "Number_pb2"
        # @@protoc_insertion_point(class_scope:tzrpc.proto.FloatArrayList)
    },
)
_sym_db.RegisterMessage(FloatArrayList)

DoubleArrayList = _reflection.GeneratedProtocolMessageType(
    "DoubleArrayList",
    (_message.Message,),
    {
        "DESCRIPTOR": _DOUBLEARRAYLIST,
        "__module__": "Number_pb2"
        # @@protoc_insertion_point(class_scope:tzrpc.proto.DoubleArrayList)
    },
)
_sym_db.RegisterMessage(DoubleArrayList)


# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# @Time  : 2021/7/25 13:18
# @Author : lovemefan
# @Email : lovemefan@outlook.com
# @File : BertBase.py
from abc import ABCMeta, abstractmethod


class BertBase(metaclass=ABCMeta):
    def __init__(self, name: str):
        self.name = name

    def __str__(self):
        return f"bert for {self.name}"

    @abstractmethod
    def output(self, input: str):
        pass



# -*- coding: utf-8 -*-
# @Time  : 2021/8/17 9:47
# @Author : lovemefan
# @Email : lovemefan@outlook.com
# @File : rpc.py
import pickle

from tzrpc.proto.py.Boolean_pb2 import Boolean
from tzrpc.proto.py.Bytes_pb2 import Bytes
from tzrpc.proto.py.Number_pb2 import Double, Float, Integer
from tzrpc.proto.py.Numpy_pb2 import ndarrays
from tzrpc.proto.py.String_pb2 import String
from tzrpc.utils.logger import get_logger
from tzrpc.utils.numpy_serialized import numpy2protobuf, protobuf2numpy

servicers = []

logger = get_logger(to_std=True, stdout_level="INFO", save_log_file=False)


class RpcServicer:
    def __init__(self):
        pass

    def register(self, task):
        _listener = Listener(task)
        # print(_listener)
        logger.debug(f"Instance's task  {_listener.task} registered")
        servicers.append(_listener)

        def wrapper(*args, **kwargs):
            print(args, kwargs)
            print(f"{task.__name__}: text")
            self.args = args
            self.kwargs = kwargs
            return task(*args, **kwargs)

        return wrapper


class Listener:
    def __init__(self, task):
        logger.info(f"Method {task} {task.__name__}() registered.")
        self.task = task
        logger.debug(f"self Method {self.task} {task.__name__}() registered.")

    def wrapper(self, *args, **kwargs):
        print(args, kwargs)
        print(f"{self.task.__name__}: text")
        return self.task(*args, **kwargs)

    def toString(self, request, context):
        logger.debug(f"Method toString({request}) called.")
        result = self.task(request.text)
        response = String(text=result)
        return response

    def toInteger(self, request, context):
        logger.debug(f"Method toInteger({request}, {context}) called.")
        result = self.task(request.value)
        response = Integer(value=result)
        return response

    def toFloat(self, request, context):
        logger.debug(f"Method toFloat({request}, {context}) called.")
        result = self.task(request.value)
        response = Float(value=result)
        return response

    def toDouble(self, request, context):
        logger.debug(f"Method toDouble({request}, {context}) called.")
        result = self.task(request.value)
        response = Double(value=result)
        return response

    def toBoolean(self, request, context):
        logger.debug(f"Method toBoolean({request}, {context}) called.")
        result = self.task(request.value)
        response = Boolean(value=result)
        return response

    def toBytes(self, request, context):
        logger.debug(f"Method toBytes({request}, {context}) called.")
        data = request.data
        is_pickled = True if b"PICKLE" in data[:6] else False

        if is_pickled:
            data = pickle.loads(data[6:])
        result = self.task(data)
        if is_pickled:
            result = pickle.dumps(result)
        response = Bytes(data=result)
        return response

    def toNdarray(self, request, context):
        logger.debug(f"Method toNdarray({request}) called.")
        value = protobuf2numpy(request)
        data = self.task(value)
        response = numpy2protobuf(data)
        return response

    def toNdarrays(self, request, context):
        logger.debug(f"Method toNdarrays({request}) called.")
        _ndarray = self.task(request.value)
        response = ndarrays(ndarray=_ndarray)
        return response

    def toTensor(self, request, context):
        pass

    def toTensors(self, request, context):
        pass

    def toIntegerArrayList(self, request, context):
        pass

    def toFloatArrayList(self, request, context):
        pass

    def toDoubleArrayList(self, request, context):
        pass

    def toBooleanArrayList(self, request, context):
        pass



# -*- coding: utf-8 -*-
# @Time  : 2021/7/24 10:14
# @Author : lovemefan
# @Email : lovemefan@outlook.com
# @File : RPCServer.py
import logging
import os
import traceback
from asyncio import get_event_loop
from concurrent import futures
from typing import Optional

import grpc

from tzrpc.decorator.rpc import servicers
from tzrpc.exceptions.exceptions import TZRPCException
from tzrpc.proto.py.Server_pb2_grpc import add_toObjectServicer_to_server
from tzrpc.server.base import tzrpcBase
from tzrpc.utils.constant import MAX_MESSAGE_LENGTH, MAX_METADATA_SIZE
from tzrpc.utils.logger import get_logger

logger = get_logger(to_std=True, stdout_level="INFO", save_log_file=False)


class TZRPC_Server(tzrpcBase):
    def __init__(self, name, debug=False):
        if debug:
            get_logger(to_std=True, stdout_level="DEBUG" if debug else "INFO", save_log_file=False)\
                .setLevel(logging.DEBUG)

        super().__init__(name)

    @property
    def loop(self):
        """
        Synonymous with asyncio.get_event_loop().
        .. note::
            Only supported when using the `app.run` method.
        """
        if not self.is_running:
            raise TZRPCException(
                "Loop can only be retrieved after the app has started "
                "running. Not supported with `create_server` function"
            )
        return get_event_loop()

    def run(
        self,
        host: Optional[str] = None,
        port: Optional[int] = None,
        debug: bool = False,
        workers: int = 1,
        loop: None = None,
    ) -> None:
        """
        Run the HTTP Server and listen until keyboard interrupt or term
        signal. On termination, drain connections before closing.
        :param host: Address to host on
        :type host: str
        :param port: Port to host on
        :type port: int
        :param debug: Enables debug output (slows server)
        :type debug: bool
        :param workers: Number of processes received before it is respected
        :type workers: int
        :return: Nothing
        """
        if loop is not None:
            raise TypeError(
                "loop is not a valid argument. To use an existing loop, "
                "change to create_server().\nSee more: "
                "https://sanic.readthedocs.io/en/latest/sanic/deploying.html"
                "#asynchronous-support"
            )

        host, port = host or "127.0.0.1", port or 8000
        get_logger(to_std=True, stdout_level="DEBUG" if debug else "INFO", save_log_file=False)
        try:
            self.is_running = True
            self.is_stopping = False
            if workers > 1 and os.name != "posix":
                logger.warn(
                    f"Multiprocessing is currently not supported on {os.name},"
                    " using workers=1 instead"
                )
                workers = 1
            if workers == 1:
                logger.info(f"Tzrpc Server now listening {host}:{port}.")
                server = grpc.server(futures.ThreadPoolExecutor(),
                                     options=[('grpc.max_send_message_length', MAX_MESSAGE_LENGTH),
                                      ('grpc.max_receive_message_length', MAX_MESSAGE_LENGTH),
                                              ('grpc.max_metadata_size', MAX_METADATA_SIZE)]
                                     )
                for servicer in servicers:
                    add_toObjectServicer_to_server(
                        servicer, server, servicer.task.__name__
                    )

                server.add_insecure_port(f"{host}:{port}")
                server.start()
                logger.info("Tzrpc Server started.")
                server.wait_for_termination()
            else:
                # TODO
                raise ValueError(
                    "it so sorry for mutil-process is not available, please set work = 1"
                )
        except BaseException:
            traceback.print_exc()
        finally:
            self.is_running = False
        logger.info("Server Stopped")

    def stop(self):
        """
        This kills the tzrpc
        """
        if not self.is_stopping:
            self.is_stopping = True
            get_event_loop().stop()



# -*- coding: utf-8 -*-
# @Time  : 2021/8/6 11:20
# @Author : lovemefan
# @Email : lovemefan@outlook.com
# @File : setup.py
import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

requirements = {"install": ["grpcio==1.40.0", "grpcio-tools==1.40.0", "numpy"]}

setuptools.setup(
    name="tzrpc",
    version="v0.0.2",
    author="Lovemefan, Yunnan Key Laboratory of Artificial Intelligence, "
    "Kunming University of Science and Technology, Kunming, Yunnan ",
    author_email="lovemefan@outlook.com",
    description="a rpc framework for python base on grpc",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/lovemefan/TZRPC",
    packages=setuptools.find_namespace_packages(),
    install_requires=requirements["install"],
    python_requires=">=3.7.0",
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



# -*- coding:utf-8 -*-
# @FileName  :logger.py
# @Time      :2023/7/26 09:42
# @Author    :lovemefan
# @Email     :lovemefan@outlook.com
import logging
import logging.handlers
import os
import sys

loggers_dict = {}
LEVEL = ("DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL")
DEFAULT_LOG_FILE_DIR = "~/.cache/tzrpc-log/"
LOCAL_DEFAULT_LOG_FILE_DIR = os.path.join(
    os.getenv("LOCAL_DEFAULT_PATH", DEFAULT_LOG_FILE_DIR), "log"
)
DEFAULT_FILEHANDLER_FORMAT = (
    "[%(levelname)s] %(asctime)s " "[%(pathname)s:%(lineno)d] %(funcName)s: %(message)s"
)
DEFAULT_STDOUT_FORMAT = (
    "%(asctime)s - %(name)s - %(levelname)s - ["
    "%(filename)s:%(lineno)d] %(funcName)s: - %(message)s"
)
DEFAULT_REDIRECT_FILE_NAME = "tzrpc.log"


def _convert_level(level: str) -> int:
    """Convert the format of the log to logging level.

    Args:
        level (str): User log level.

    Returns:
        level (str): Logging level.
    """
    level_convert = {
        "DEBUG": logging.DEBUG,
        "INFO": logging.INFO,
        "WARNING": logging.WARNING,
        "ERROR": logging.ERROR,
        "CRITICAL": logging.CRITICAL,
    }
    level = level_convert.get(level, logging.INFO)

    return level


def get_logger(logger_name: str = "tzrpc", **kwargs) -> logging.Logger:
    """Get the logger. Both computing centers and bare metal servers are
    available.

    Args:
        logger_name (str): Logger name.
        kwargs (dict): Other input.
            to_std (bool): If set to True, output the log to stdout.
            save_log_file (bool): If set to false, output will not write a log file.
            stdout_level (str): The level of the log output to stdout.
                If the type is str, the options are DEBUG, INFO, WARNING, ERROR, CRITICAL.
            stdout_format (str): Log format.
            file_level (list[str] or tuple[str]): The level of the log output to file.
                eg: ['INFO', 'ERROR'] Indicates that the logger will output logs above
                    the level INFO and ERROR in the list to the corresponding file.
                The length of the list needs to be the same as the length of file_name.
            file_save_dir (str): The folder where the log files are stored.
            file_name (list[str] or list[tuple]): Store a list of output file names.
            max_file_size (int): The maximum size of a single log file. Unit: MB.
            max_num_of_files (int): The maximum number of files to save.

    Returns:
        logger (logging.Logger): Logger.
    """

    if logger_name in loggers_dict:
        return loggers_dict[logger_name]
    mf_logger = logging.getLogger(logger_name)

    save_log_file = kwargs.get("save_log_file", True)
    stdout_level = kwargs.get("stdout_level", "INFO")
    stdout_format = kwargs.get("stdout_format", "")
    file_level = kwargs.get("file_level", ("INFO", "ERROR"))
    file_save_dir = kwargs.get("file_save_dir", "")
    file_name = kwargs.get("file_name", ("info.log", "error.log"))
    max_file_size = kwargs.get("max_file_size", 50)
    max_num_of_files = kwargs.get("max_num_of_files", 5)

    to_std = kwargs.get("to_std", True)
    if to_std:
        if not stdout_format:
            stdout_format = DEFAULT_STDOUT_FORMAT
        stream_handler = logging.StreamHandler(sys.stdout)
        stream_handler.setLevel(_convert_level(stdout_level))
        stream_formatter = logging.Formatter(stdout_format)
        stream_handler.setFormatter(stream_formatter)
        mf_logger.addHandler(stream_handler)

    logging_level = []
    for level in file_level:
        logging_level.append(_convert_level(level))

    if save_log_file:
        if not file_save_dir:
            file_save_dir = os.path.join(LOCAL_DEFAULT_LOG_FILE_DIR, "log")

        file_path = []
        for name in file_name:
            path = os.path.join(file_save_dir, name)
            path = os.path.realpath(path)
            base_dir = os.path.dirname(path)
            if not os.path.exists(base_dir):
                os.makedirs(base_dir, exist_ok=True)
            file_path.append(path)

        max_file_size = max_file_size * 1024 * 1024

        file_formatter = logging.Formatter(DEFAULT_FILEHANDLER_FORMAT)
        for i, level in enumerate(file_level):
            file_handler = logging.handlers.RotatingFileHandler(
                filename=file_path[i],
                maxBytes=max_file_size,
                backupCount=max_num_of_files,
            )
            file_handler.setLevel(level)
            file_handler.setFormatter(file_formatter)
            mf_logger.addHandler(file_handler)

    mf_logger.propagate = False
    mf_logger.parent = None

    loggers_dict[logger_name] = mf_logger

    return mf_logger
